module Elasticmusic
  class Song
    include Elasticsearch::Persistence::Model
    include ActiveModel::Validations::Callbacks

    attribute :path       , String
    attribute :tag        , Hash
    attribute :properties , Hash

    before_validation :validate_file

    def set_metatags
      fileref = TagLib::FileRef.new(path)

      unless fileref.null?
        self.tag        = tags(fileref)
        self.properties = audio_properties(fileref)
        fileref.close
      end
    end

    private

      def tags(fileref)
        {
          title:  fileref.tag.title,
          artist: fileref.tag.artist,
          album:  fileref.tag.album,
          genre:  fileref.tag.genre,
          track:  fileref.tag.track,
          year:   fileref.tag.year
        }
      end

      def audio_properties(fileref)
        {
          bitrate:     fileref.audio_properties.bitrate,
          channels:    fileref.audio_properties.channels,
          length:      fileref.audio_properties.length,
          sample_rate: fileref.audio_properties.sample_rate
        }
      end

      def validate_file
        if File.exists?(path.to_s)
          if !Elasticmusic::SUPPORTED_FORMATS.include?(File.extname(path))
            errors.add(:path, "The format #{File.extname(path).downcase} is not suported")
          end
        else
          errors.add(:path, 'The specified path is not valid')
        end
      end
  end
end